# coding: utf-8

"""
Servizio CodemaOnline
=====================

Modulo per spedizione e ricezione di fax dal servizio codemaonline_ che
prevede:

* una casella pop3 cui invia la notifica di un fax in arrivo
* una spedizione di un allegato pdf (altri formati ammessi) attraverso un
  suo servizio (per ora non abilitato)

.. autoclass:: CodemaMsg
   :members:

.. autofunction:: get_messages

.. autofunction:: get_msg

.. _codemaonline: http://www.codemaonline.com/index.php/fax-virtuale.html

Esempi di messaggi ricevuti sono in tests/data


FIXME: Occorre cancellare i messaggi dopo averli letti. Forse occorre tenere
       traccia dell'id di quelli letti
"""

import re
import email
import poplib
import datetime

def get_messages(host, user, pass_, key_file=None, cert_file=None, limit=100):
    """Return all messages from host with user/pass

    :param host: the pop3 host
    :param user: the user 
    :param pass_: the password
    :param key_file: a possible key (extension .pem)
    :param cert_file: a possible certificate (extension .pem)
    :param limit: e max number of messages to retrieve
    """
    
    if key_file:
        M = poplib.POP3_SSL(host, 995, key_file, cert_file)
        M.user(user)
    else:
        M = poplib.POP3(host)
        M.user(user)
        
    M.pass_(pass_)
    msg_count, mailbox_size = M.stat()
    return [get_msg(M, i+1) for i in range(min(msg_count, limit))]

        
def get_msg(pop3_instance, n):
    """Return an ``email.message.Message``

    :param pop3_instance: a poplib.POP3 instance
    :param n: the msg number
    """
    # FIXME: supposes mailbox doesn not change in between
    # Result is in form (response, ['line', ...], octets).
    ret = pop3_instance.retr(n)
    
    return email.message_from_string("\n".join(ret[1]))

class CodemaMsg(object):
    """
    Il messaggio ricevuto ha un body primario ed un allegato
    """
    fax_content = None
    "The fax content"

    status = None
    "Status del messaggio: RECEIVED, SENT, FAILED_RECEIVED, FAILED_SENT"
    
    info = None
    """un dict con chiavi tali da poter usare Fax(**info):
    sender_number, sender, date FIXME: date non esiste nel modello"""

    def __init__(self, email_msg):
        """parse the message

        :param email_msg: an ``email.message.Message`` instance retrieved from pop
        """
        
        self.msg = email_msg
        payload_count = len(self.msg.get_payload())
        if payload_count == 2:
            # notification + attached fax
            self.notification_msg, self.attach_msg = self.msg.get_payload()
        elif payload_count == 1:
            # presumbly just notification
            self.notification_msg = self.msg.get_payload()[0]
        else:
            raise NotImplementedError('Too many payload...')

        self.parse_notification()
        
    def parse_notification(self):
        """Parse the notification to extract info
        le notifiche di ricezione corretta recano::

           E' stato ricevuto un nuovo fax.

        le notifiche di NON spedizione::

           ERRORE: FAX NON INVIATO - Nessun circuito/canale disponibile
           
        """
        if re.search("E' stato ricevuto un nuovo fax.", self.notification):
            self.status = 'RECEIVED'
            info = {}
            for line in self.notification.split('\n'):
                try:
                    key, value = line.split(': ')
                except:
                    continue
                if key == 'Numero Mittente':
                    info['sender_number'] = value.strip()
                elif key == 'Nome Mittente':
                    info['sender'] = value.strip()
                elif key == 'Data/ora ricez':
                    info['date'] = datetime.datetime.strptime(value.strip(), '%Y/%m/%d %H:%M:%S')
            self.info = info
                    
            
        elif re.search("FAX NON INVIATO", self.notification):
            self.status = 'FAILED_SENT'

    @property
    def notification(self):
        "The text of the notification received from the fax gateway"
        return self.notification_msg.get_payload()

    @property
    def attach(self):
        "The received fax as attached pdf document"
        return self.attach_msg.get_payload(decode=True)

    def write(self, file_name=None):
        """Write the attachment to file
        :param file_name: a possible filename (default to attach_msg..get_filename())
        :return: file_name
        """
        file_name = file_name or self.attach_msg.get_filename()
        with open(file_name, 'wb') as f:
            f.write(self.attach)
        return file_name

    def __str__(self):
        if self.status == 'RECEIVED':
            return "<Fax %s (%s %s)>" % (self.status, self.info['sender_number'],
                                         self.info.get('date', ''))
        else:
            return "<Fax %s>" % self.status
        
