import datetime, re

from django import forms
from django.utils.translation import ugettext as _
from django.forms.util import ErrorList

from jumbo.forms.models import ModelForm
from jumbo.apps.autocomplete import AutocompleteMultiChoiceField
import jumbo.middleware.thread_local as tl
from jumbo.utils.functions import clean_phone_number
from jumbo.utils.functions import get_settings

from fax.models import Fax, FaxNumber, Cover, Device, STATUS_CODES


class CoverFormInline(ModelForm):
    class Meta:
        model  = Cover
        fields = ('societa_to', 'societa_from', 'to_person',
                  'from_person', 'pag', 'oggetto', 'note')


class FaxForm(ModelForm):
    class Meta:
        model  = Fax
        fields = ('subject', 'submitter', 'notify_group', 'read', 'keywords',)


class FaxSendForm(ModelForm):
    class Meta:
        model  = Fax
        fields = ('priority', 'subject', 'keywords', 'file')

    def __init__(self, *args, **kwargs):
        super(FaxSendForm, self).__init__(*args, **kwargs)

        # Se la cover e' abilitato do la possibilita di mandare un fax solo
        # immettendo i dati della cover. Cosi facendo la cover non viene
        # inviata come cover ma come file del fax (e' stata una richiesta di
        # Co.ge.ma che volevano inviare dei fax senza allegato ma solo con
        # le informazioni immesse nella cover)
        ENABLE_COVER = get_settings('fax.settings', 'ENABLE_COVER')
        if ENABLE_COVER:
            #self.fields['file'].required = False
            pass
        # find devices available for the user to send faxes
        d = FaxNumber.objects.all()
        initial = None
        if len(d) == 1:
            initial = d[0].id
        self.fields['device'] = forms.ModelChoiceField(queryset=d, initial=initial, required=True, label=_("N. Fax"))
        self.fields['recipient_numbers'] = AutocompleteMultiChoiceField('fax_organization', label=_("recipient number"), force_selection=False)


    def clean_recipient_numbers(self):
        # Controllo tutti i numeri di telefono inseriti dall'utente
        #
        for number in self.cleaned_data['recipient_numbers']:
            number = number.split("-")[-1].strip()
            # pulisco il numero di telefono da caratteri strani
            number = clean_phone_number(number)

            if not number.isdigit():
                raise forms.ValidationError(_("One ore more numbers inserted are wrong."))
        return self.cleaned_data['recipient_numbers']


    def save_and_send(self, formset=None, fax_to_forward=None):
        request = tl.get_request()
        lista_fax = []

        for i, number in enumerate(self.cleaned_data['recipient_numbers']):
            name_and_number = number.split('-')

            # Recupero il numero e lo tolgo dalla lista, in modo che mi rimanga solo l'eventuale nome
            #
            number = name_and_number.pop().strip()

            # pulisco il numero di telefono da caratteri strani
            number = clean_phone_number(number)

            name = None
            if name_and_number:
                name = '-'.join(name_and_number).strip()

            # Il primo fax lo salvo tramite il meccanismo di django.
            # In questo modo la gestione e il relativo salvataggio del file di upload la
            # gestisce lui.
            #
            if i == 0:
                # Se sto forwardando un fax creo un fax nuovo, non modifico il
                # precedente
                if fax_to_forward:
                    first_fax = Fax()
                    first_fax.subject = self.cleaned_data['subject']
                    first_fax.priority = self.cleaned_data['priority']
                    first_fax.keywords = self.cleaned_data['keywords']
                    first_fax.device = self.cleaned_data['device'].device.all()[0]
                    first_fax.file = fax_to_forward.file
                else:
                    first_fax = super(FaxSendForm, self).save(commit=False)
                first_fax.recipient = name
                first_fax.recipient_number = number
                first_fax.submitter = request.user
                first_fax.read = 1
                first_fax.status = 3
                first_fax.device = self.cleaned_data['device'].device.all()[0]
                first_fax.save()
                lista_fax.append(first_fax.id)
            # Per il salvataggio di tutti gli altri fax evito di gestire di nuovo il file
            # ma ne recupero solo il percorso dal fax salvato per primo
            else:
                other_fax = Fax(
                    subject = first_fax.subject,
                    priority = first_fax.priority,
                    keywords = first_fax.keywords,
                    recipient = name,
                    read = 1,
                    status = 3,
                    recipient_number = number,
                    submitter = request.user,
                    device = first_fax.device,
                    file = first_fax.file,
                )

                other_fax.save()
                lista_fax.append(other_fax.id)

        for id in lista_fax:
            f = Fax.objects.get(pk=id)
            f.send_fax(formset)

class FaxScanForm(FaxSendForm):
    class Meta:
        model = Fax
        fields = ('priority', 'subject', 'keywords',)
