# -*- coding: utf-8 -*-

import os, time, datetime
import random

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.utils.translation import ugettext as ugt
from django.contrib.auth.models import User, Group, Permission
from django.core.files.storage import FileSystemStorage
from django.db.models import Q
from django.db import connection
from django.core.exceptions import PermissionDenied
from django.contrib.contenttypes.models import ContentType
from django.template.defaultfilters import slugify

from jumbo.db.models import DateModel, UserModel, StatusModel, OrderedModel
from jumbo.middleware.thread_local import get_current_user
from jumbo.utils.functions import get_settings
from jumbo.db.models import Settings as JSettings

import commands

STATUS_CODES = (
    (2, ugt('Received')),
    (3, ugt('Queued')),
    (4, ugt('Sent')),
    (5, ugt('Failed')),
    (6, ugt('Multi Queued')),
    (7, ugt('Multi Sent')),
    (8, ugt('To Send')),
)

PRIORITIES = (
    (127, ugt('Low')),
    (120, ugt('Normal')),
    (100, ugt('High')),
    (50, ugt('Urgent')),
    )

#########################
# FAX                   #
#########################
class FaxManager(models.Manager):
    def get_query_set(self):
        user = get_current_user()
        groups = user.groups.all()

        qs = super(FaxManager, self).get_query_set()

        # FIXME : Questo if serve per lo script che chiama hylafax.
        if not user or user.is_anonymous():
            return qs

        if user.has_perm("fax.list_all_fax"):
            return qs

        q = Q(is_public=True) | \
                  (Q(is_public=False) & Q(submitter=user)) | \
                  (Q(is_public=False) & Q(notify_group__in=user.groups.all()))

        if user.has_perm("fax.list_all_fax_not_assigned"):
            q1 = Q(is_public=False)
            q2 = Q(notify_group__isnull=True)
            q = q | (q1 & q2)

        list_lperms = FaxPermission.objects.filter(codename__startswith='list')
        # Controllo quali numeri di fax l'utente puo' visualizzare
        #
        for perm in list_lperms:
            if user.has_perm("fax.%s" % perm.codename):
                # Recupero, a partire dai numeri di telefono associati ai permessi,
                 #i device (le tty) che l'utente puo vedere.
                # Questo perche da hylafax viene passato il device ed e' quello che
                # noi andiamo a salvare nel modello dei fax.
                try:
                    allow_device = [ device.name for device in Device.objects.filter(numbers__number = perm.number) ]
                except Exception, e:
                    return qs.none()

                qdevice = Q(device__in = allow_device)

                # Puo' vedere tutti i fax di quel numero?
                if perm.codename.find("list_all_faxes_from") >= 0:
                    q = q | qdevice

                # Puo' vedere solo quelli non assegnati?
                elif perm.codename.find("list_all_faxes_not_assigned_from") >= 0:
                    q1 = (qdevice & Q(notify_group=None))
                    q = q | q1

        return qs.filter(q)

FAX_SENT_BY = (
    ("W", ugt('WEB')),
    ("M", ugt('MAIL')),
)

class Fax (DateModel):
    status = models.IntegerField (
        max_length = 3,
        default = '3',
        choices = STATUS_CODES,
        verbose_name = _("status"),
    )
    job = models.CharField (
        blank = True,
        null = True,
        max_length = 15,
        verbose_name = _("job"),
        help_text = _("job help text"),
    )
    keywords = models.CharField (
        blank = True,
        null = True,
        max_length = 100,
        verbose_name = _("keywords"),
    )
    pages = models.PositiveSmallIntegerField (
        blank = True,
        null = True,
        max_length = 3,
        verbose_name = _("pages"),
    )
    date_required = models.DateTimeField (
        blank = True,
        null = True,
        verbose_name = _("date required"),
        help_text = _("date required help text"),
    )
    date_sent = models.DateTimeField (
        blank = True,
        null = True,
        verbose_name = _("date sent"),
    )
    ## submitter/receiver
    submitter = models.ForeignKey (
        User,
        blank = True,
        null = True,
        verbose_name = _("submitter"),
    )
    ## group that can view the fax
    notify_group = models.ForeignKey (
        Group,
        blank = True,
        null = True,
        verbose_name = _("notify group"),
    )
    recipient = models.CharField (
        blank = True,
        null = True,
        max_length = 100,
        verbose_name = _("recipient"),
    )
    recipient_number = models.CharField (
        max_length = 16,
        verbose_name = _("recipient number"),
    )
    sender = models.CharField (
        blank = True,
        null = True,
        max_length = 100,
        verbose_name = _("sender"),
    )
    sender_number = models.CharField (
        blank = True,
        null = True,
        max_length = 16,
        verbose_name = _("sender number"),
    )
    duration = models.CharField (
        blank = True,
        null = True,
        max_length = 10,
        verbose_name = _("duration"),
    )
    device = models.CharField (
        blank = True,
        null = True,
        max_length = 10,
        verbose_name = _("device"),
    )
    file = models.FileField (
        null = True,
        upload_to = get_settings("fax.settings", "FAX_PATH_DOCUMENTS"),
        verbose_name = _("file"),
    )
    error = models.CharField (
        blank = True,
        null = True,
        max_length = 150,
        verbose_name = _("error"),
    )
    subject = models.CharField (
        max_length = 100,
        verbose_name = _("subject"),
    )
    cover = models.BooleanField (
        default = False,
        verbose_name = _("cover"),
    )
    read = models.IntegerField (
        null = True, blank = True,
        choices = ((0,_('Not Read')), (1,_('Read'))),
        default = 0,
        verbose_name = _("read"),
    )
    priority = models.PositiveSmallIntegerField (
        blank = True,
        null = True,
        default = 127,
        verbose_name = _("priority"),
        #verbose_name = _("Priority"),
        choices = PRIORITIES
    )
    sent_by = models.CharField (
        max_length = 1,
        verbose_name = _("Sent By"),
        choices = FAX_SENT_BY,
        default = 'W'
    )

    objects = FaxManager()

    is_public = models.BooleanField(
        default = False,
    )

    def send_fax(self, formset=None):
        ENABLE_COVER = get_settings("fax.settings", "ENABLE_COVER")
        if ENABLE_COVER:
            from jumbo.views.generic.create_oo_file import create_file
            #from jumbo.utils.ooutils import OOUtils, DocumentConverter
        from jumbo.middleware.thread_local import get_request
        import tempfile
        from django.template.loader import get_template
        from django.template import TemplateDoesNotExist

        ###################################
        # CONTROLLO E CREAZIONE COVER
        ###################################
        has_cover = False
        # Se un campo della richiesta cover e' compilato aggiungo la cover al fax
        if formset and formset.cleaned_data[0]:
            societa_to = formset.cleaned_data[0]['societa_to']
            societa_from = formset.cleaned_data[0]['societa_from']
            to_person = formset.cleaned_data[0]['to_person']
            from_person = formset.cleaned_data[0]['from_person']
            fax = ''
            pages = formset.cleaned_data[0]['pag']
            object_fax = formset.cleaned_data[0]['oggetto']
            note = formset.cleaned_data[0]['note']
            date_cover = datetime.datetime.today()
            date_f = date_cover.strftime("%d/%m/%Y %H:%M")

            path_modello = None
            if ENABLE_COVER:
                request = get_request()

                pdf_cover_name = "odt_file_%s_%s.pdf" % (time.time(), random.randint(0,99))
                ps_cover_name  = "/tmp/ps_file_%s_%s.ps" % (time.time(), random.randint(0,99))

                extra_context = {'societa_to':societa_to, 'societa_from':societa_from,
                                 'to_person':to_person, 'from_person':from_person,
                                 'fax':fax, 'pages':pages, 'object_fax':object_fax,
                                 'date_f':date_f, 'note':note}

                str_oofile = create_file(request, file_name=pdf_cover_name, file_model='fax/cover/cover.odt',
                                        file_type="pdf", save_in="/tmp/", extra_context=extra_context)

                cmd = "pdf2ps -sPAPERSIZE=a4 %s %s" % ('/tmp/'+pdf_cover_name, ps_cover_name)
                status, output = commands.getstatusoutput(cmd)

                has_cover = True

        ###################################
        # FINE CONTROLLO E CREAZIONE COVER
        ###################################

        if not self.recipient_number:
            return

        if self.file:
            ffname = self.file.path
        else:
            ffname = '/tmp/'+pdf_cover_name

        # Converto, per sicurezza, il file pdf in un a4 verticale
        # Infatti, quando si scanerrizza dalla stampante capita che il pdf creato
        # sia verticale ma con la page size a 842x596, invece di 596x842, e questo
        # fa si che il fax non venga spedito
        #
        info = {
            'path'       : '/usr/bin/sendfax',
            'start_time' : 'now',
            'fax_number' : self.recipient_number,
            'file_name'  : ffname, #self.file.path, #'/tmp/%s' % (ps_tmp_file,),
            'priority'   : self.priority or 127,
            'device'     : self.device + "@127.0.0.1"
            }

        cmd = "sudo %(path)s -a %(start_time)s -P %(priority)s"

        # Aggiungo effettivamente la cover come cover se anche un file e'
        # stato aggiunto, altrimenti la cover verra' spedita come file
        if has_cover and self.file:
            cmd = cmd + " -C " + ps_cover_name
        else:
            cmd = cmd + " -n"

        cmd = cmd + " -s a4 -B 9600 -D -d %(fax_number)s -h %(device)s %(file_name)s"
        cmd = cmd % info

        status, output = commands.getstatusoutput(cmd)

        if has_cover:
            #os.remove(odt_cover_name)
            os.remove('/tmp/'+pdf_cover_name)
            os.remove(ps_cover_name)

        if status:
            raise Exception

        ## output from sendfax:
        ## request id is 879 (group id 879) for host localhost (1 file)
        self.job = output.split()[3]
        self.save()

        # Se e' presente una cover la salvo
        if formset and formset.cleaned_data[0]:
            fmset = formset.save(commit=False)
            fmset[0].date_fax = date_cover
            fmset[0].save()

    def mark_as_read(self):
        self.read = 1

    class Meta:
        verbose_name = _("Fax")
        verbose_name_plural = _("Fax")
        permissions = (
            ("view_fax"                       , "Can view fax"),
            ("download_all_report"            , "Can view all reports"),
            ("download_all_document"          , "Can download all documents"),
            ("send_all_fax"                   , "Can send all faxes"),
            ("list_all_fax"                   , "Can list all faxes"),
            ("list_all_fax_not_assigned"      , "Can list all faxes not assigned"),
            ("delete_all_fax_to_send"         , "Can Delete ALL FAX TO SEND"),
            ("delete_all_own_fax_to_send"     , "Can Delete ALL OWN FAX TO SEND"),
            ("delete_all_owngroup_fax_to_send", "Can Delete ALL OWN GROUP FAX TO SEND"),
            ("delete_all_own_fax"             , "Can Delete ALL OWN FAX"),
            ("delete_all_owngroup_fax"        , "Can Delete ALL OWN GROUP FAX"),
        )
        ordering = ('-date_create',)


    def can_be_delete(self):
        from jumbo.middleware.thread_local import get_current_user
        user = get_current_user()
        user_group = [ g.name for g in user.groups.all() ]
        list_perms = user.get_all_permissions()

        ## controlla permessi di delete "ALL" o "STAFF"
        if user.is_superuser or "fax.delete_fax" in list_perms:
            return True

        # Se il fax e' in stato TO SEND controllo altri tipi
        # di permessi
        if self.status == 8:
            if "fax.delete_all_fax_to_send" in list_perms:
                return True

            if "fax.delete_all_owngroup_fax_to_send" in list_perms and \
                self.notify_group and self.notify_group.name in user_group:
                return True

            if "fax.delete_all_own_fax_to_send" in list_perms and \
                self.submitter == user:
                return True

        # Controllo permessi di cancellazione dei proprio fax
        if "fax.delete_all_owngroup_fax" in list_perms and \
           self.notify_group and self.notify_group.name in user_group:
            return True
        if "fax.delete_all_own_fax" in list_perms and \
           self.submitter == user:
            return True

        list_lperms = FaxPermission.objects.filter(codename__startswith='delete')
        # Controllo quali numeri di fax l'utente puo' visualizzare
        # perm.model_id rappresenta il device
        #
        for perm in list_lperms:
            if user.has_perm("fax.%s" % perm.codename):
                # Se il device del fax non e' lo stesso del permesso salto a prescindere
                # tt i controlli
                try:
                    allow_device = [ device.name for device in Device.objects.filter(numbers__number = perm.number) ]
                except Exception, e:
                    return False

                if self.device in allow_device:
                    if perm.codename.find("delete_all_fax_from") >= 0:
                        return True
                    if perm.codename.find("delete_own_fax_from") >= 0:
                        if self.submitter == user:
                            return True
                    if perm.codename.find("delete_owngroup_fax_from") >= 0:
                        if self.notify_group and self.notify_group.name in user_group:
                            return True

        return False


    def delete(self):
        if not self.can_be_delete():
            raise PermissionDenied
        super(Fax, self).delete()


    def __unicode__(self):
        return unicode(self.subject)


SEND_MODE = (
    (1, 'Fax'),
    (2, 'Email'),
)


class SendFax(DateModel):
    date_required = models.DateTimeField (
        blank = True,
        null = True,
        verbose_name = _("date required"),
        help_text = _("date required help text"),
        #verbose_name = _("Ora spedizione"),
        #help_text = _("Ora richiesta di spedizione, default: subito."),
    )

    submitter = models.ForeignKey (
        User,
        blank = True,
        null = True,
        verbose_name = _('submitter')
        #verbose_name = _('Inviato da')
    )

    status = models.IntegerField (
        max_length = 3,
        default = '3',
        choices = STATUS_CODES,
        verbose_name = _("status"),
        #verbose_name = _("Stato"),
    )

#    groups = models.ManyToManyField(
#        OrganizationGroup,
#        blank = True,
#        null = True,
#    )

#    organizations = models.ManyToManyField(
#        Organization,
#        blank = True,
#        null = True,
#    )

    recipient_numbers = models.CharField (
        blank = True,
        null = True,
        max_length = 255,
        verbose_name = _("recipient numbers"),
        help_text = _("recipient numbers help text"),
        #verbose_name = _("Destinatari non in anagrafica"),
        #help_text = _("Separati dal carattere 'punto e virgola'."),
    )

    mode = models.PositiveSmallIntegerField(choices = SEND_MODE)

    subject = models.CharField (
        max_length = 100,
        verbose_name = _("subject"),
        #verbose_name = _("Oggetto"),
    )

    keywords = models.CharField (
        blank = True,
        null = True,
        max_length = 100,
        verbose_name = _("keywords"),
        #verbose_name = _("Parole chiave ricerca"),
    )

    forward = models.IntegerField(
        default = 0,
    )

    priority = models.PositiveSmallIntegerField (
        blank = True,
        null = True,
        default = 127,
        verbose_name = _("priority"),
        #verbose_name = _("Priority"),
        choices = PRIORITIES
    )

    file = models.FileField (
        null = True,
        upload_to = _("fax"),
        verbose_name = _("file"),
        #verbose_name = _("File"),
    )

    class Meta:
        pass

    def save(self):
        pass

class Cover(DateModel):
    fax = models.OneToOneField (
        Fax,
        primary_key = True,
        related_name = 'cover_fax'
    )
    societa_to = models.CharField(
        blank = True, null = True,
        verbose_name = _("To"),
        max_length = 100
    )
    societa_from = models.CharField(
        blank = True, null = True,
        verbose_name = _("From"),
        max_length = 100
    )
    to_person = models.CharField(
        blank = True, null = True,
        verbose_name = _("To Person"),
        max_length = 100
    )
    from_person = models.CharField(
        blank = True, null = True,
        verbose_name = _("From Person"),
        max_length = 100
    )
    num_fax = models.CharField(
        blank = True, null = True,
        verbose_name = _("Fax number"),
        max_length = 30
    )
    pag = models.CharField(
        blank = True, null = True,
        verbose_name = _("Pag."),
        help_text = _("Cover Include"),
        max_length = 2
    )
    oggetto = models.CharField(
        blank = True, null = True,
        verbose_name = _("Object"),
        max_length = 100
    )
    date_fax = models.DateTimeField(
        blank = True, null = True,
        verbose_name = _("Date")
    )
    note = models.TextField(
        blank = True, null = True,
        verbose_name = _("Note")
    )
    class Meta:
        verbose_name = _("Cover")
        verbose_name_plural = _("Cover")


class Device(models.Model):
    name = models.CharField(
        max_length = 100,
        verbose_name = _("Name"),
        unique = True
    )
    description = models.CharField(
        max_length = 200,
        blank = True, null = True,
        verbose_name = _("Description")
    )

    def __unicode__(self):
        return "%s" % (self.name,)


class FaxNumberManager(models.Manager):
    def get_query_set(self):
        user = get_current_user()
        qs = super(FaxNumberManager, self).get_query_set()

        if user.is_superuser or user.has_perm("fax.send_all_fax"):
            return qs

        list_lperms = FaxPermission.objects.filter(codename__startswith='send')

        q = Q()
        # Controllo da quali numeri di fax l'utente puo' inviare
        #
        for perm in list_lperms:
            if user.has_perm("fax.%s" % perm.codename):
                q = q | Q(number = perm.number)

        return qs.filter(q)

class FaxNumber(DateModel, UserModel, StatusModel, OrderedModel):
    device = models.ManyToManyField(
        Device,
        related_name = "numbers",
        verbose_name = _("Device")
    )
    number = models.CharField(
        max_length = 30,
        verbose_name = _("Number"),
        db_column = "dnumber",
        unique = True
    )
    name = models.CharField(
        max_length = 100,
        verbose_name = _("Name"),
        blank = True, null = True
    )
    notify_mails = models.CharField(
        max_length = 200,
        blank = True, null = True,
        verbose_name = _("Notify Mails")
    )
    send_at_received_fax = models.BooleanField()
    send_at_sent_fax = models.BooleanField()
    objects = FaxNumberManager()


    def __unicode__(self):
        if self.name:
            return "%s - %s" % (self.number, self.name)
        return self.number


    class Meta:
        verbose_name = _("Fax Number")
        verbose_name_plural = _("Fax Numbers")
        ordering = ['ordering',]


    def save(self, *args, **kwargs):
        if not self.id:
            content_type = ContentType.objects.get_for_model(self)
            super(FaxNumber, self).save(*args, **kwargs)
            for perm in get_settings('fax.settings', 'DEFAULT_PERMS'):
                permission = FaxPermission()
                permission.name = perm[1] % self.number
                permission.codename = perm[0] % slugify(self.number)
                permission.content_type = content_type
                permission.number = self.number
                permission.save()
        else:
            old_number = FaxNumber.objects.get(pk = self.pk).number
            super(FaxNumber, self).save(*args, **kwargs)
            if old_number != self.number:
                for perm in get_settings('fax.settings', 'DEFAULT_PERMS'):
                    f = FaxPermission.objects.get(codename = perm[0] % slugify(old_number))
                    f.name = perm[1] % self.number
                    f.codename = perm[0] % slugify(self.number)
                    f.save()


    def delete(self, *args, **kwargs):
        for perm in get_settings('fax.settings', 'DEFAULT_PERMS'):
            f = FaxPermission.objects.get(codename = perm[0] % slugify(self.number)).delete()
        super(FaxNumber, self).delete(*args, **kwargs)


############################
# FAX PERMISSIONS          #
############################
class FaxPermission(Permission):
    number = models.CharField(
        max_length = 30,
        verbose_name = _("Number"),
        db_column = "dnumber",
    )


############################
# FAX SETTINGS             #
############################
class Settings(JSettings):
    pass
