from django.conf import settings
from django import forms
from django.contrib.auth.decorators import login_required
from django.utils import simplejson
from django.http import HttpResponse, HttpResponseRedirect, HttpResponseNotFound, HttpResponseForbidden
from django.core.files.uploadedfile import SimpleUploadedFile
from django.utils.translation import ugettext as _
from django.forms.models import modelformset_factory, inlineformset_factory

from jumbo.shortcuts import render_to_response, render_to_string
from jumbo.core.urlresolvers import reverse
from jumbo.utils.functions import get_settings

from fax.models import Fax, Cover, FaxPermission
from fax.forms import FaxSendForm, FaxForm, FaxScanForm, CoverFormInline


def check_send_permission(user):
    if user.has_perm("fax.send_all_fax"):
        return True

    list_lperms = FaxPermission.objects.filter(codename__startswith='send')
    for perm in list_lperms:
        if user.has_perm("fax.%s" % perm.codename):
            return True

    return False


def fax_change(request, object_id):
    fax = Fax.objects.get(pk = object_id)

    if request.method == 'POST':
        form = FaxForm(request.POST, request.FILES, instance=fax)
        if form.is_valid():
            form.save()
            return HttpResponseRedirect(reverse('admin:fax_fax_changelist'))
    else:
        form = FaxForm(instance=fax)

    return render_to_response('admin/fax/fax/user_change_form.html', request=request, form=form, app_label='fax')


def fax_send(request):
    if not check_send_permission(request.user):
        return HttpResponseForbidden(render_to_string('admin/forbidden.html'))

    formset = None
    ENABLE_COVER = get_settings('fax.settings', 'ENABLE_COVER')
    if ENABLE_COVER:
        CoverFormset = inlineformset_factory(Fax, Cover, form=CoverFormInline, fk_name='fax', extra=1, can_delete=False)

    fax = Fax()

    if request.method == 'POST':
        form = FaxSendForm(request.POST, request.FILES, instance=fax)
        if ENABLE_COVER:
            formset  = CoverFormset(request.POST, request.FILES, instance=fax)
            if form.is_valid() and formset.is_valid():
                # Se la COVER e' abilitata contorlla che sia stato rimepito o il campo
                # del file o almeno uno dei campi della cover
                if ENABLE_COVER:
                    is_valid = True
                    if not form.cleaned_data['file']:
                        is_valid = False
                        # Controllo che almeno un campo della cover sia stato inserito
                        for key in formset.forms[0].cleaned_data.keys():
                            if formset.forms[0].cleaned_data[key]:
                                is_valid = True

                    if not is_valid:
                        form._errors['file'] = form.error_class([_("Insert a file or fill the cover fields.")])
                        del form.cleaned_data['file']
                        return render_to_response('admin/fax/fax/send.html', form=form, formset=formset,
                                                  ENABLE_COVER=ENABLE_COVER, app_label='fax')

                form.save_and_send(formset)
                return HttpResponseRedirect(reverse('admin:fax_fax_changelist'))
        else:
            if form.is_valid():
                form.save_and_send()
                return HttpResponseRedirect(reverse('admin:fax_fax_changelist'))

    else:
        form = FaxSendForm(instance=fax)
        if ENABLE_COVER:
            formset  = CoverFormset(instance=fax)

    return render_to_response('admin/fax/fax/send.html', form=form, formset=formset,
                              ENABLE_COVER=ENABLE_COVER, app_label='fax')


def fax_forward(request, object_id):
    if not check_send_permission(request.user):
        return HttpResponseForbidden(render_to_string('admin/forbidden.html'))

    formset = None
    ENABLE_COVER = get_settings('fax.settings', 'ENABLE_COVER')
    if ENABLE_COVER:
        CoverFormset = inlineformset_factory(Fax, Cover, form=CoverFormInline, fk_name='fax', extra=1, can_delete=False)

    fax = Fax.objects.get(pk=object_id)

    if request.method == 'POST':
        form     = FaxScanForm(request.POST, request.FILES, instance=fax)
        if ENABLE_COVER:
            formset  = CoverFormset(request.POST, request.FILES, instance=fax)
            if form.is_valid() and formset.is_valid():
                form.save_and_send(formset=formset, fax_to_forward=fax)
                return HttpResponseRedirect(reverse('admin:fax_fax_changelist'))
        else:
            if form.is_valid():
                form.save_and_send(fax_to_forward=fax)
                return HttpResponseRedirect(reverse('admin:fax_fax_changelist'))
    else:
        form     = FaxScanForm(instance=fax)
        if ENABLE_COVER:
            formset  = CoverFormset(instance=fax)

    return render_to_response('admin/fax/fax/send.html', form=form, formset=formset,
                              ENABLE_COVER=ENABLE_COVER, fax=fax, app_label='fax')


def download_pdf(request, object_id):
    from django.views.static import serve

    #if request.user.has_perm("fax.download_document"):
    try:
        r = Fax.objects.get(id=object_id)
        if not r.file:
            raise Exception
        # cambio lo stato da Non letto a letto
        r.mark_as_read()
        r.save()
        return serve(request, r.file.path, '/')
    except Exception, e:
        return HttpResponseNotFound(render_to_string('fax/error_report.html', error=e))
#    else:
#        return HttpResponseForbidden(render_to_string('jumbo/forbidden.html'))


@login_required
def report_pdf(request, object_id):
    from reportlab.pdfgen import canvas
    from django.http import HttpResponse

#    if not request.user.has_perm("fax.view_reports"):
#        return HttpResponseForbidden(render_to_string('jumbo/forbidden.html'))

    f = Fax.objects.get(pk=object_id)

    response = HttpResponse(mimetype='application/pdf')
    response['Content-Disposition'] = 'attachment; filename=report_%s.pdf' % f.id

    p = canvas.Canvas(response)

    date_fax = f.date_last_modify.strftime('%d/%m/%Y %H:%M:%S')
    date_sent = f.date_sent.strftime('%d/%m/%Y %H:%M:%S')

    if f.status == 2:
        p.drawString(30, 800, _('Sender: %s' % f.sender))
        p.drawString(30, 780, _('Job ID: %s' % f.job))
        p.drawString(30, 760, _('Received Date: %s' % date_fax))
        p.drawString(30, 740, _('Received Time: %s' % f.duration))
        p.drawString(30, 720, _('Received Pages: %s' % f.pages))

    elif f.status in (4, 5): #sent or failed
        if f.status == 4:
            confirm = 'OK'
        else:
            confirm = f.error

        dest = "%s" % f.recipient_number
        if f.recipient:
            dest = "%s (%s)" % (f.recipient, f.recipient_number)

        p.drawString(30, 800, _('Recipient: %s' % dest))
        p.drawString(30, 780, _('Subject: %s' % f.subject))
        p.drawString(30, 760, _('Job ID: %s' % f.job))
        p.drawString(30, 740, _('Sent Date: %s' % date_sent))
        p.drawString(30, 720, _('Sent Confirm: %s' % confirm))
        p.drawString(30, 700, _('Sent Time: %s' % f.duration))
        p.drawString(30, 680, _('Sent Pages: %s' % f.pages))

    p.showPage()
    p.save()

    return response


def download_cover(request, object_id):
    from jumbo.views.generic.create_oo_file import create_file
    #from jumbo.utils.ooutils import OOUtils
    #from django.template.loaders.app_directories import get_template_sources

    if request.user.has_perm("fax.add_cover"):
        try:
            r = Fax.objects.get(id=object_id)
            #if not r.file:
            #    raise Exception
            if not r.cover_fax:
                return render_to_string(
                    'fax/error_report.html',
                    error="Non e' stata inviata nessuna cover con questo fax"
                )
        except Exception, e:
            return HttpResponseNotFound(render_to_string('fax/error_report.html', error=e))
    else:
        return HttpResponseForbidden(render_to_string('jumbo/forbidden.html'))

    extra_context = {
        'societa_to':r.cover_fax.societa_to, 'societa_from':r.cover_fax.societa_from,
        'to_person':r.cover_fax.to_person, 'from_person':r.cover_fax.from_person,
        'fax':r, 'pages':r.cover_fax.pag, 'object_fax':r.cover_fax.oggetto,
        'date_f':r.cover_fax.date_fax, 'note':r.cover_fax.note
    }

    return create_file(
        request, file_name='cover', file_model='fax/cover/cover.odt',
        file_type="pdf", extra_context=extra_context
    )